@@ -1,6 +1,6 @@
 package VCS::CMSynergy::Object;
 
-our $VERSION = do { (my $v = q$Revision: 325 $) =~ s/^.*:\s*//; $v };
+our $VERSION = do { (my $v = q$Revision: 342 $) =~ s/^.*:\s*//; $v };
 
 =head1 NAME
 
@@ -123,9 +123,9 @@ sub is_dir	{ return shift->cvtype eq "dir"; }
 sub is_project	{ return shift->cvtype eq "project"; }
 
 
-# NOTE: All access to a VCS::CMSynergy::Objects data must either use
+# NOTE: All access to a VCS::CMSynergy::Objects data _must_ either use
 # methods, e.g. "$self->foo", or use _private(), e.g. "$self->_private->{foo}".
-# Don't access its member directly, e.g. "$slef->{data}", becaus this
+# _Don't_ access its member directly, e.g. "$self->{foo}", because this
 # doesn't work when :tied_objects are enabled.
 # The only exception to this rule are the primary getter methods (objectname,
 # version etc) which use direct access for speed. Hence they need to be
@@ -134,6 +134,13 @@ sub is_project	{ return shift->cvtype eq "project"; }
 # access to private parts
 sub _private 	{ return shift; }
 
+sub mydata
+{
+    my ($self) = @_;
+    return $self->_private->{mydata} ||= {};
+}
+
+
 sub list_attributes
 {
     my ($self) = @_;
@@ -298,7 +305,13 @@ sub cvid
     return $self->_private->{cvid} ||= $self->property('cvid');
 }
 
-
+sub cat_object
+{
+    my $self = shift;
+    # NOTE: careful here to correctly handle the case when 
+    # no destination was given
+    return $self->ccm->cat_object($self, @_);
+}
 
 # $obj->is_foo_of: short for $ccm->query_object({is_foo_of => [ $obj ]})
 # same for has_foo
@@ -417,6 +430,20 @@ is C<"project"> or C<"dir">, resp.
 C<ccm> returns the session (a C<VCS::CMSynergy>) that is associated
 with the object.
 
+=head2 cat_object
+
+  $contents = $obj->cat_object();
+  $obj->cat_object($destination);
+
+A convenience wrapper for L<VCS::CMSynergy/cat_object>.
+
+=head2 mydata
+
+Sometimes it is handy to be able to store some arbitrary data 
+in a C<VCS::CMSynergy::Object>. This method returns a reference
+to a hash associated with the object. It is totally opaque
+w.r.t. Synergy operations. 
+
 =head1 ATTRIBUTE METHODS
 
 =head2 get_attribute, set_attribute
